package com.fossgalaxy.games.ggj2017;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.util.Collection;

import org.codetome.hexameter.core.api.CubeCoordinate;
import org.codetome.hexameter.core.api.Hexagon;

import com.fossgalaxy.games.rts.GameState;
import com.fossgalaxy.games.rts.entity.Entity;
import com.fossgalaxy.games.rts.entity.HexagonTile;
import com.fossgalaxy.games.rts.io.SpriteRegistry;
import com.fossgalaxy.games.rts.order.MoveOrder;
import com.fossgalaxy.games.rts.order.Order;
import com.fossgalaxy.games.rts.ui.GameAction;
import com.fossgalaxy.games.rts.ui.GameView;

public class MoveAction implements GameAction {
	private final Color HINT_MOVE_COLOUR = new Color(161, 191, 253, 50);
	private final Stroke HINT_STROKE = new BasicStroke(2f);
	
	@Override
	public void renderHints(Graphics2D g, GameState s, Entity actor) {
		if (actor == null) {
			return;
		}
		
		if (!JamApp.isZoneOfControl(s, actor)) {
			SpriteRegistry sr = SpriteRegistry.INSTANCE;
			BufferedImage img = sr.getImage("ui/no-signal");
			
			Hexagon<HexagonTile> ht = s.cube2hex(actor.getPos());
			
			g.drawImage(img, (int)ht.getCenterX() - 25, (int)ht.getCenterY() - 25, 50, 50, null);
			
			return;
		}
		
		Collection<Hexagon<HexagonTile>> range = s.getRange(actor.getPos(), actor.getType().getMovement());
		
		Area area = new Area();
		
		range.forEach(h -> {
	        if (!isPossible(actor, s, h.getCubeCoordinate())) {
	        	return;
	        }
	        
		    Shape hexShape = GameView.hex2shape.apply(h);
	        area.add(new Area(hexShape));
	        
	        g.setColor(HINT_MOVE_COLOUR);
	        g.fill(hexShape);
		});
		
        g.setColor(Color.BLUE);
        g.setStroke(HINT_STROKE);
        g.draw(area);
		
	}


    @Override
	public boolean isPossible(Entity entity, GameState s, CubeCoordinate co) {
    	
    	//movement is not possible if the space is occupied
		Entity e = s.getEntityAt(co);
		if (e != null) {
			return false;
		}
		
		//movement is not possible if it's too far away
    	int movementDist = entity.getType().getMovement();
    	if (movementDist < s.getDistance(entity.getPos(), co)) {
    		return false;
    	}
    	
		//movement might not be possible
		return isPossible(entity);
	}
	
	@Override
	public Order generateOrder(CubeCoordinate co, GameState s) {
		return new DroneMovement(co);
	}

	@Override
	public boolean isPossible(Entity entity) {
		return entity.getType().getMovement() >= 1;
	}


	@Override
	public Color getHintColour() {
		return HINT_MOVE_COLOUR;
	}


	@Override
	public Color getBorderColour() {
		return Color.BLUE;
	}
}
