package com.fossgalaxy.games.ggj2017;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.util.Collection;
import java.util.List;

import org.codetome.hexameter.core.api.CubeCoordinate;
import org.codetome.hexameter.core.api.Hexagon;

import com.fossgalaxy.games.rts.GameState;
import com.fossgalaxy.games.rts.entity.Entity;
import com.fossgalaxy.games.rts.entity.HexagonTile;
import com.fossgalaxy.games.rts.io.SpriteRegistry;
import com.fossgalaxy.games.rts.order.AttackOrderRanged;
import com.fossgalaxy.games.rts.order.MoveOrder;
import com.fossgalaxy.games.rts.order.Order;
import com.fossgalaxy.games.rts.ui.GameAction;
import com.fossgalaxy.games.rts.ui.GameView;

public class RangeAttackAction implements GameAction {
	private final Color HINT_ATTACK_COLOUR = new Color(229, 5, 26, 50);
	private final Stroke HINT_STROKE = new BasicStroke(2f);
	
	@Override
	public void renderHints(Graphics2D g, GameState s, Entity actor) {
		if (actor == null) {
			return;
		}
		
		if (!JamApp.isZoneOfControl(s, actor)) {
			SpriteRegistry sr = SpriteRegistry.INSTANCE;
			BufferedImage img = sr.getImage("ui/no-signal");
			
			Hexagon<HexagonTile> ht = s.cube2hex(actor.getPos());
			
			g.drawImage(img, (int)ht.getCenterX() - 25, (int)ht.getCenterY() - 25, 50, 50, null);
			
			return;
		}
		
		Collection<Hexagon<HexagonTile>> range = s.getRange(actor.getPos(), actor.getType().getAttackRange());
		System.out.println(range);
		
		Area area = new Area();
		
		range.forEach(h -> {
	        Entity e = s.getEntityAt(h.getCubeCoordinate());
	        if (e == null) {
	        	return;
	        }
	        
	        //don't attack friendlies - it's rude
	        if (e.getOwner() == actor.getOwner()) {
	        	return;
	        }
	        
		    Shape hexShape = GameView.hex2shape.apply(h);
	        
	        area.add(new Area(hexShape));
	        
	        g.setColor(HINT_ATTACK_COLOUR);
	        g.fill(hexShape);
	        
	        g.setColor(Color.RED);
	        g.setStroke(HINT_STROKE);
	        g.draw(hexShape);
		});
		
	}

	@Override
	public Order generateOrder(CubeCoordinate co, GameState s) {
		Entity e = s.getEntityAt(co);
		if (e == null) {
			return null;
		}
		
		return new AttackOrderRanged(e);
	}

	
	
	@Override
	public boolean isPossible(Entity entity, GameState s, CubeCoordinate co) {
    	//movement is not possible if the space is occupied
		Entity e = s.getEntityAt(co);
		if (e == null) {
			return false;
		}
		
		if (e.getOwner() == entity.getOwner()) {
			return false;
		}
		
		//movement is not possible if it's too far away
    	int movementDist = entity.getType().getAttackRange();
    	if (movementDist < s.getDistance(entity.getPos(), co)) {
    		return false;
    	}
    	
		//movement might not be possible
		return isPossible(entity);
	}

	@Override
	public boolean isPossible(Entity entity) {
		return entity.getType().getAttackRange() >= 1;
	}

	@Override
	public Color getHintColour() {
		return HINT_ATTACK_COLOUR;
	}

	@Override
	public Color getBorderColour() {
		// TODO Auto-generated method stub
		return Color.RED;
	}
}
